#ifndef _LOGRECORDIMPL_H
#define _LOGRECORDIMPL_H

#include <string>
#include <time.h>
#include <unistd.h>
#include <strstream.h>
#include "LogRecord.h"
#include "LogRecordSpec.h"
#include "Buffer.h"

/* GENCODE START
#include <string>
enum LogRecordType
{
	LogData,
	LogString,
	LogInt,
	LogLong,
	LogNV,
};

enum Criticality
{
	CRITICAL,
	MAJOR,
	MINOR,
	INFO
};
 GENCODE END */

extern char* logStrings[];

class OutputPairs {};
class OutputData {};

class LogRecordHdr
{
public:
	LogRecordHdr( Criticality crit, unsigned int type, unsigned int tag, unsigned int comp, unsigned int file, unsigned int line ) : m_Component(comp), m_File(file), m_Line(line), m_Criticality(crit), m_Type(type), m_Tag(tag)
	{
		gettimeofday(&m_Time, NULL);
		m_Pid = getpid();
	}
	LogRecordHdr(char* loadData) 
	{
		memcpy(this, loadData, sizeof(*this));
	}

	char* getType()					{ return logStrings[m_Type-1];		}
	char* getTag()					{ return logStrings[m_Tag-1];		}
	string& getCriticality()			{ static string s("INFO"); return s;	}
	char* getComponent()				{ return logStrings[m_Component-1];	}
	char* getFile()					{ return logStrings[m_File-1];		}
	int getLine()					{ return m_Line;			}
	int getPid()					{ return m_Pid;				}
	unsigned int getSize()				{ return sizeof(*this);			}
	string  getTime()					
	{ 
		char tbuff1[1024],tbuff2[1024];
		cftime( tbuff1, "%m/%d/%y %T", &m_Time.tv_sec );
		strstream str(tbuff2,sizeof(tbuff2));
		str << m_Time.tv_usec/10000 << ends;
		return (string)tbuff1 + " " + str.str();
	}

	void store(char* dest)
	{
		memcpy( dest, this, sizeof(*this) );
	}

private:
	Criticality		m_Criticality;
	unsigned int	m_Type;
	unsigned int	m_Tag;
	unsigned int  	m_Component;
	unsigned int	m_File;
	unsigned int	m_Line;
	int				m_Pid;
	timeval			m_Time;
};

class LogRecordImpl
{
public:
	LogRecordImpl( Criticality crit, unsigned int type, unsigned int tag, unsigned int comp, unsigned int file, unsigned int line, LogRecord* ) : m_Hdr(crit,type,tag,comp,file,line), m_Buff(4096), m_Tabs(0)
	{
		m_Iterator = m_Buff.begin();
	}
	LogRecordImpl( char* loadData, unsigned int size, LogRecord* ) : m_Buff(size), m_Hdr(loadData), m_Tabs(0)
	{
		loadData += m_Hdr.getSize();
		m_Buff.load(loadData, size-m_Hdr.getSize());
		m_Iterator = m_Buff.begin();
	}
	virtual ~LogRecordImpl() 
	{
	}
	
	void insert( unsigned int name, void* data, unsigned int size )
	{
		insert( name );
		insert( LogData );
		insert( size );
		insert( data, size );
	}

	void insert( const void* data, unsigned int size )
	{
		m_Iterator = m_Buff.insert( m_Iterator, data, size );
	}

	void insert( const string& s )		{ insert( s.c_str(), s.size()+1 ); }
	void insert( LogRecordType type )	{ insert( &type, sizeof(type) ); }	
	void insert( unsigned int i )		{ insert( &i, sizeof(i) ); }
	
	string getType()			{ return m_Hdr.getType();		}
	string getTag()				{ return m_Hdr.getTag();		}
	string& getCriticality()		{ return m_Hdr.getCriticality();	}
	string getComponent()			{ return m_Hdr.getComponent();		}
	string getFile()			{ return m_Hdr.getFile();		}
	int getLine()				{ return m_Hdr.getLine();		}
	int getPid()				{ return m_Hdr.getPid();		}
	string  getTime()			{ return m_Hdr.getTime();		}	
	unsigned int getSize()			{ return m_Buff.size() + m_Hdr.getSize(); }

	void print( ostream& os )
	{
		iterator it = begin();
		
		os << getCriticality() << " " << getType() << " " << getTag() << " " << getTime() << " ProcessId: " << getPid() << endl;
		os << "File: " << getFile() << endl;
		os << "Line: " << getLine() << endl;
		os << "Component: " << getComponent() << endl;

		m_Tabs++;
		while( it != end() )
		{
			outputNVpair( os, it );
		}
		m_Tabs--;
		os << endl;
	}
	
	void store(char* dest)
	{
		m_Hdr.store(dest);
		dest += m_Hdr.getSize();
		memcpy(dest, m_Buff.get(), m_Buff.size());
	}

protected:
	typedef Buffer::iterator iterator;

	Buffer			m_Buff;
	unsigned int	m_Length;
	unsigned int	m_Tabs;
	iterator		m_Iterator;
	LogRecordHdr	m_Hdr;

	iterator begin()	{ return m_Buff.begin();	}
	iterator end()		{ return m_Buff.end();		}

	void tab( ostream& os )
	{
		for( int i=0; i < m_Tabs; i++ )
		{
			os << "	";
		}
	}

	void outputNVpair( ostream& os, iterator& it )
	{
		unsigned int iname; getVal(iname,it);
		string name(logStrings[iname-1]);
		LogRecordType type; getVal(type,it);
		tab(os);
		os << name.c_str();

		switch( type )
		{
		case LogInt:
			os << " (int): ";
			valuePrint( (int*)0, os, it );
			os << endl;
			break;
		case LogLong:
			os << " (unsigned long): ";
			valuePrint( (unsigned long*)0, os, it );
			os << endl;
			break;
		case LogString:
			os << " (string): ";
			valuePrint( (string*)0, os, it );
			os << endl;
			break;
		case LogData:
			os << " (data): ";
			valuePrint( (OutputData*)0, os, it );
			os << endl;
			break;
		case LogNV:
			getVal(name, it);
			os << " (" << name.c_str() << "): ";
			valuePrint( (OutputPairs*)0, os, it );
			break;
		}
	}

	void getVal( unsigned int& val, iterator& it )				{ memcpy( &val, it, sizeof(val) ); it += sizeof(val);	}
	void getVal( int& val, iterator& it )					{ memcpy( &val, it, sizeof(val) ); it += sizeof(val);	}
	void getVal( unsigned long& val, iterator& it )				{ memcpy( &val, it, sizeof(val) ); it += sizeof(val);	}
	void getVal( string& s, iterator& it )					{ s=static_cast<char*>(it); it += s.size()+1;		}
	void getVal( LogRecordType& rec, iterator& it )				{ memcpy( &rec, it, sizeof(rec) ); it += sizeof(rec);	}

	void valuePrint( OutputData*, ostream& os, iterator& it )	
	{
		unsigned int size; getVal( size, it );
		os << hex;
		for( int i=0; i < size; i++, it++ )
		{
			os << (int)*it;
		}
		os << dec;
	}

	void valuePrint( string*, ostream& os, iterator& it )		
	{
		unsigned int size; getVal( size, it );
		string val; getVal(val, it );
		os << val;
	}

	void valuePrint( int*, ostream& os, iterator& it )			
	{
		unsigned int size; getVal( size, it );
		int val; getVal( val, it );
		os << val;
	}

	void valuePrint( unsigned long*, ostream& os, iterator& it )			
	{
		unsigned int size; getVal( size, it );
		unsigned long val; getVal( val, it );
		os << val;
	}

	void valuePrint( OutputPairs*, ostream& os, iterator& it )	
	{
		os << endl;
		unsigned int size; getVal(size,it);
		m_Tabs++;
		for( int i=0; i < size; i++ )
		{
			outputNVpair( os, it );
		}
		m_Tabs--;
	}
};


#endif
